* ????-???-???? (LP-IOL)     DRAWING NO. 41285997-001-01
*
*
*
* COMPUTER:  H316, DDP-516
*
*
* PROGRAM CATEGORY:  INPUT/OUTPUT
*
*
* PROGRAM TITLE:  LP-IOL
*                 LINE PRINTER FORTRAN AND STANDARD INPUT/OUTPUT LIBRARY
*                 USING IOBUS
*
*
*
*
*
*
*
*
*
*
*                            APPROVAL                          DATE
*
*  
*   
*                            PROG----------------------    ------------
*
*
*                            SUPR----------------------    ------------
*
*
*                            QUAL----------------------    ------------
*
*
*                                          NO. OF PAGES
*
*
* AUTHOR   
* 
*    HONEYWELL - COMPUTER CONTROL DIVISION
*
*
* PURPOSE   
*    
*
*    TO PROVIDE A LINE PRINTER FORTRAN AND STANDARD INPUT/OUTPUT
*    LIBRARY USING IOBUS
*
*  
* RESTRICTIONS   
*   
*    THIS LIBRARY WILL WORK ON AN H316 OR DDP-516 COMPUTER WITH A
*    LINE PRINTER. THE EXACT OPTION NUMBER OF THIS LINE PRINTER IS
*    UNKNOWN. IT IS, HOWEVER, NOT THE LINE PRINTER OPTION DOCUMENTED
*    IN THE DDP-516 PROGRAMMERS REFERENCE MANUAL.
*
*    THAT OPTION (516-7050) HAS NO CHARACTER BASED STORAGE IN THE LINE
*    PRINTER CONTROL UNIT AND, AS A CONSEQUENCE, A ONE-BIT VALUE MUST
*    BE OUTPUT FOR EACH CHARACTER POSITION INDICATING WHETHER THE PRINT
*    HAMMER SHOULD FIRE FOR EACH CHARACTER POSITION AROUND THE ROTATING
*    DRUM. IN ADDITION THE DRUM ONLY INCLUDES 60 CHARACTER POSITIONS
*    AND THE PAPER IS EXPLICTLY SHUTTLED LEFT AND RIGHT, UNDER PROGRAM
*    CONTROL, TO PRINT THE ODD AND EVEN CHARATER POSITIONS ALONG THE
*    LINE.
*
*    IN CONTRAST, THE LINE PRINTER OPTION THAT THIS LIBRARY DRIVES HAS
*    STORAGE, IN THE LINE PRINTER CONTROLLER, FOR 120 CHARACTERS AND
*    THE PROGRAM MUST SIMPLY OUTPUT 60 WORDS, EACH HOLDING TWO
*    CHARACTERS, TO THE CONTROLLER FOR EACH LINE TO BE PRINTED.
*    SINCE NO MANUAL IS AVAILABLE FOR THIS OPTION, THE BEHAVIOUR OF
*    THE VARIOUS OCP, SKS, AND OTA COMMANDS HAS BEEN DEDUCED FROM THE
*    OPERATION OF THIS LIBRARY. THIS SOURCE CODE WAS RECOVERED BY
*    DISASSEMBLING THE CODE INCLUDED IN THE SELF LOADING SYSTEM TAPE
*    FOR DAP-16M2 USED ON A DDP-516 INSTALLATION.
*
*    THE LINEPRINTER OPTION THAT THIS SOFTWARE DRIVES MAY WELL BE THE
*    "LINE PRINTER OPTION 316/516-670X"  AS MENTIONED IN THE
*    "NEW AND UPDATED STANDARD SOFTWARE" NOTE OF JUNE 1971
*
* STORAGE   
*
*    SEE CONTENTS
*
*
* CONTENTS
*
*    MODULE                                              STORAGE   PAGE
*    NAME    DEFINITION                                  OCT  DEC  NO.
*
*    O$LH    LINE PRINTER HEADER INITIALIZE
*    O$LA    LINE PRINTER LIST A LINE
*    O$LE    LINE PRINTER EJECT
*    O$LLHB  LINE PRINTER LIST HEADING BUFFER
*    O$LLHW  LINE PRINTER LIST HEADING BUFFER NO. OF WORDS
*    O$LLPN  LINE PRINTER LIST PAGE NUMBER
*    O$LLLC  LINE PRINTER LIST LINE COUNT/PAGE           407  263
*
*
* USE
*   
*    INTERNAL MODULES  
*
*    NAME    CALLING SEQUENCE              CALLS   ERRORS
*
*    O$LH    CALL  O$LH                            NONE
*            DAC   HEADING BUFFER
*            RETURN
*
*    O$LA    CALL  O$LA                    O$LE    NONE
*            DAC   LINE BUFFER
*            RETURN
*
*    O$LE    CALL  O$LE                            NONE
*            RETURN
*
*
*    EXTERNAL MODULES
*
*      NONE
*
*
* ERRORS   
*
*    NONE ARE REPORTED IN A STANDARDIZED MANNER. IF AN ALARM IS
*    DETECTED, UP TO TEN ATTEMPTS ARE MADE TO OUTPUT THE LINE, IF
*    THAT FAILS THE PROGRAM HALTS WITH 052525 IN THE A-REGISTER.
*
*
* METHOD   
*
*    O$LH SETS UP A HEADING LINE WHICH IS STORED IN A BUFFER PROVIDED
*    BY THIS MODULE (O$LHB). THE DEFAULT NUMBER OF WORDS COPIED IS
*    48 (96 CHARACTERS), HOWEVER THE O$LLHW MAY BE MODIFIED TO SET A
*    DIFFERENT NUMBER OF WORDS. NORMALLY, THE PAGE NUMBER WILL BE
*    PRINTED AFTER THE HEADING SUPPLIED BY O$LH AT THE TOP OF EACH
*    PAGE. IN ORDER TO AVOID A PARTIAL PAGE NUMBER, IF O$LLHW IS
*    GREATER THAN 55 (110 CHARACTERS) THEN IT IS MODIFIED TO THE
*    VALUE 60, AND 60 WORDS ARE COPIED INTO THE BUFFER TO BE PRINTED
*    AT THE TOP OF EACH PAGE WITH NO PAGE NUMBER.
*
*    O$LA PRINTS A LINE. IF THE CURRENT PAGE IS FULL WHEN THIS ROUTINE
*    IS CALLED THEN O$LE IS AUTOMATICALLY CALLED TO MOVE TO THE START
*    OF THE NEXT PAGE. THE NUMBER OF LINES PRINTED PER PAGE IS
*    INDICATED BY O$LLLC. THIS VALUE MUST BE SET CONSIDERING THE SIZE
*    OF THE PAPER FORM AND THE PLACEMENT OF THE VERTICAL TAB STOPS.
*
*    O$LE EJECTS TO THE START OF THE NEXT PAGE. THE HEADING IS PRINTED
*    WITH THE PAGE NUMBER, AND THE PAGE NUMBER IS INCREMENTED. THE
*    PRINTER ADVANCES TO THE FIRST VERTICAL TAB STOP IN READINESS FOR
*    PRINTING THE FIRST LINE.
*
\EJCT
*
* NOTES ON RECOVERY FROM DISASSEMBLY
*
*    THE NAMES O$LH, O$LA, AND O$LE ARE KNOWN TO BE CORRECT SINCE
*    THEY ARE CALLED FROM THE IOS-O16X DEVICE SELECTION DRIVER OF
*    DAP-16M2, FOR WHICH SOURCE CODE IS AVAILABLE.
*
*    THE NAMES O$LLHB, O$LLHW, O$LLPN, O$LLC, AND O$LLLN ARE NOT
*    KNOWN, AND IT IS POSSIBLE (EVEN LIKELY) THAT THESE LOCATIONS
*    HAD DIFFERENT NAMES IN THE ORIGINAL SOURCE CODE, OR THAT THEY
*    HAD NO GLOBAL NAMES VISIBLE TO THE LOADER. THE NAMES ARE CHOSEN
*    BY ANALOGY WITH THE SIMILAR FUNCTIONS IN THE ASR AND PAPER TAPE
*    ROUTINES OF SIMILAR FUNCTIONALITY. O$LLHW, ALMOST CERTAINLY HAD
*    A GLOBAL (LOADER) NAME SINCE IF IT CANNOT BE MODIFIED IT MAKES
*    NO SENSE TO COMPARE THIS VALUE TO LITERALS AS THIS CODE DOES.
*    (HOWEVER, SINCE THERE IS NO EQUIVALENT IN THE ASR AND PAPER TAPE
*    ROUTINES IT IS EVEN MORE LIKELY THAT ITS NAME IS INCORRECT.)
*
*    THESE THREE ROUTINES (O$LH, O$LA, AND O$LE) MUST HAVE BEEN
*    CONTAINED IN A SINGLE ASSEMBLER SOURCE MODULE SINCE THEY ARE
*    COVERED BY A SINGLE LITERAL POOL. (THEY COULD NOT HAVE BEEN
*    SEPARATELY ASSEMBLED AND LINKED BY THE LOADER).
*
*    THE DRAWING NUMBER (41285997-001-01) IS TAKEN FROM THE ORDER
*    FORM OF THE "NEW AND UPDATED STANDARD SOFTWARE" NOTE OF JUNE 1971
*    THOUGH THAT REVISION NUMBER MAY WELL NOT CORRESPOND TO THIS
*    CODE.
*
\EJCT
*
*
\SUBR\O$LH\LINE PRINTER HEADER INITIALIZE
\SUBR\O$LA\LINE PRINTER LIST A LINE
\SUBR\O$LE\LINE PRINTER EJECT
*
\SUBR\O$LLHB,HEAD\LINE PRINTER LIST HEADING BUFFER
\SUBR\O$LLHW,HWRD\LINE PRINTER LIST HEADING BUFFER NO. OF WORDS
\SUBR\O$LLPN,PAGE\LINE PRINTER LIST PAGE NUMBER
\SUBR\O$LLLC,PAGE\LINE PRINTER LIST LINE COUNT/PAGE
\SUBR\O$LLLN,PAGE\LINE PRINTER LIST LINE NUMBER
*
*
\CF5\\H316 AND DDP-516
\REL\\RELOCATABLE
*
*
* LINE PRINTER DEVICE CODES
*
* LINE PRINTER ADDRESS
LPTA\EQU\'3\LINE PRINTER ADDRESS
*
* OCP
LPSS\EQU\LPTA+'0100\LINE PRINTER START SCAN
LPES\EQU\LPTA+'0200\LINE PRINTER END SCAN
LPFF\EQU\LPTA+'1300\LINE PRINTER FORM FEED
LPVT\EQU\LPTA+'1600\LINE PRINTER SKIP TO VERTICAL TAB
LPLN\EQU\LPTA+'1700\LINE PRINTER STEP ONE LINE
*
* SKS
LPSR\EQU\LPTA+'0000\LINE PRINTER SKIP IF READY
LPNB\EQU\LPTA+'0200\LINE PRINTER SKIP IF NOT BUSY
LPNA\EQU\LPTA+'0300\LINE PRINTER SKIP IF NO ALARM
LPNP\EQU\LPTA+'0400\LINE PRINTER SKIP IF NOT SHIFTING PAPER
*
* OTA
LPCP\EQU\LPTA+'0000\LINE PRINTER OUTPUT CHARACTER PAIR
*
*
O$LH\DAC\**\HEADER INITIALIZE
\LDA*\O$LH\PICK UP BUFFER POINTER
\STA\ADDR\AND SAVE
*
\LDA\HDRA\HEADER BUFFER TO BE INITIALIZED
\STA\HPTR\PLACE IN POINTER
\LDA\=-66\-VE SIZE OF BUFFER
\STA\CNTR\INITIALIZE COUNTER
\LDA\=A  \TWO SPACES
CLLP\STA*\HPTR\STORE IN BUFFER
\IRS\HPTR\STEP POINTER
\IRS\CNTR\STEP COUNTER
\JMP\CLLP\GO BACK UNTIL BUFFER FILLED
*
\LDA\HDRA
\STA\HPTR\POINT TO START OF BUFFER
\LDA\=55\(110 CHARACTERS)
\SUB\HWRD\NUMBER OF WORDS IN HEADER
\SMI\\MORE THAN 110 CHAR.S?
\JMP\HCPY\NO COPY JUST HWRD
\LDA\=60\SET TO 60
\STA\HWRD\COPY WHOLE BUFFER
HCPY\CRA
\SUB\HWRD\A = -HRWD
\JST\MCPY\COPY TEXT ACROSS
\LDA\PGNA\PAGE NUMBER BUFFER
\STA\ADDR\SET ADDRESS
\LDA\=-3\2 WORDS FOR 'PAGE' ONE SPACE EACH SIDE
\JST\MCPY\COPY INTO HEADER
\LDA\HPTR\1ST TWO CHARS OF PAGE NUM. GO HERE
\ADD\=1\ADD ONE BECAUSE
\STA\HPN2\2ND TWO CHARS OF PAGE NUM. GO HERE
\LDA\LCNT\LIST LINE COUNT/PAGE
\ADD\=1\START BEYOND END OF PAGE
\STA\LINE\TO FORCE INITIAL NEW PAGE
\LDA\=1\RESET CURRENT PAGE NUMBER
\STA\PAGE\TO ONE
\IRS\O$LH\SKIP PARAMETER
\JMP*\O$LH\RETURN
*
MCPY\DAC\**\COPY MEMORY FROM 'ADDR'
\STA\CNTR\-VE NO. WORDS IN A ON ENTRY
MCPL\LDA*\ADDR\PICK UP WORD
\STA*\HPTR\MORE TO NEW LOCATION
\IRS\ADDR\STEP SOURCE ADDR
\IRS\HPTR\STEP DEST ADDR
\IRS\CNTR\STEP COUNTER
\JMP\MCPL\LOOP FOR NEXT WORD
\JMP*\MCPY\ALL DONE - RETURN
*
*
* VARIABLES AND CONSTANTS
*
CNTR\BSS\1\COUNTER
ADDR\BSS\1\ADDRESS POINTER
*
* LIST HEADER WORDS
HWRD\DEC\48\WORDS IN HEADER (96 CHAR. DEFAULT)
*
* LIST LINE COUNT/PAGE
LCNT\DEC\50\LIST LINE COUNT/PAGE
*
* LIST PAGE NUMBER
PAGE\DEC\1\LIST PAGE NUMBER
*
HDSZ\DAC\HDRE-HEAD\MAXIMUM NO. WORDS IN HEADER
HPTR\DAC\HEAD+51\1ST TWO CHARS OF PAGE NUM. GO HERE
HDRA\DAC\HEAD\ADDRESS OF HEADER BUFFER
HPN2\DAC\HEAD+52\2ND TWO CHARS OF PAGE NUM. GO HERE
PGNA\DAC\PAGN\POINT AT ' PAGE ' TEXT
*
* LIST LINE NUMBER
LINE\OCT\77777\MAXIMUM +VE INT. - FORCE NEW PAGE
*
DECI\BSS\4\BUFFER FOR ASCII CONV. PAGE NUMBER
*
\DEC\1000\TABLE OF DECIMAL POWERS
\DEC\100
\DEC\10
DTAB\EQU\*
*
*
O$LA\DAC\**\OUTPUT LINE TO LINEPRINTER
\STX\SAVX\SAVE
\LDA\LCNT\LIST LINE COUNT/PAGE
\SUB\LINE\CURRENT LINE
\SPL\\ANY SPACE THIS PAGE?
\JST\O$LE\NO - EJECT
\IRS\LINE\STEP LINE NUMBER
\SKS\LPNP\SKIP IF NOT SHIFTING PAPER
\JMP\*-1
\OCP\LPLN\STEP ONE LINE
\LDA*\O$LA\ADDRESS OF BUFFER TO PRINT
\JST\OUTL
\IRS\O$LA
\JMP*\O$LA
*
*
O$LE\DAC\**\EJECT
\STX\SAVX\SAVE
\CRA\\CLEAR...
\STA\DECI\1000S DIGIT
\STA\DECI+1\100S DIGIT
\STA\DECI+2\10S DIGIT
*
\LDA\=9999\MAX 4-DIGIT DECIMAL
\SUB\PAGE\CURRENT VALUE
\SMI\\TOO BIG?
\JMP\DECC\NO, IT'S OK - GO CONVERT
\CRA\\YES, RESET TO ZERO
\STA\PAGE
*
* CONVERT TO DECIMAL
DECC\LDA\PAGE\PICK UP PAGE NUMBER
\LDX\=-3\3 POWERS OF 10
DECL\SUB\DTAB,1\SUBTRACT A POWER OF TEN
\SPL\\STILL POSITIVE?
\JMP\DECM\NO, IT'S GONE -VE
\IRS\DECI+3,1\YES, INCREMENT ASCII DIGIT
\JMP\DECL\SUBTRACT POWER AGAIN
DECM\ADD\DTAB,1\OOPS, -VE - RESTORE IT
\IRS\0\STEP COUNTER
\JMP\DECL\LOOP FOR NEXT SMALLER POWER OF 10
\STA\DECI+3\REMAINDER IS UNITS
*
\IRS\PAGE\INCREMENT FOR NEXT PAGE
*
\LDA\DECI\PICK UP 1000S
\SZE\\ZERO?
\JMP\D100\NO - SO TWO CHARS IN FIRST WORD
\LDA\='020000\ONE SPACE IN 1ST CHAR
\STA*\HPTR\SAVE IN BUFFER
\LDA\DECI+1\PICK UP 100S
\SZE\\ZERO?
\JMP\D101\NO - ONE CHAR
\LDA\='020040\YES, TWO SPACES
\STA*\HPTR\PLACE IN BUFFER
\LDA\DECI+2\PICK UP 10S
\SNZ\\ZERO?
\ADD\=-'20\YES, LATER +'60 GOES TO SPACE
\JMP\D102
*
D100\ADD\='60\CONVERT TO ASCII
\LGL\8\TO UPPER CHAR
\STA*\HPTR\PLACE IN BUFFER
\LDA\DECI+1\PICK UP 100S
D101\ADD\='60\CONVERT TO ASCII
\ADD*\HPTR\ADD IN UPPER CHARACTER
\STA*\HPTR\PLACE PAIR IN BUFFER
\LDA\DECI+2\PICK UP 10S
*
D102\ADD\='60\CONVERT TO ASCII
\LGL\8\TO UPPER CHAR
\ADD\DECI+3\ADD IN UNITS
\ADD\='60\CONVERT TO ASCII
\STA*\HPN2\PLACE PAIR IN BUFFER
*
\SKS\LPNP\SKIP IF NOT SHIFTING PAPER
\JMP\*-1
\OCP\LPFF\FORM FEED
*
\LDA\HDRA\POINT AT HEADER BUFFER
\JST\OUTL\AND PRINT
*
\SKS\LPNP\SKIP IF NOT SHIFTING PAPER
\JMP\*-1
\OCP\LPVT\SKIP TO VERTICAL TAB
*
\LDA\=1\SET CURRENT LINE NUMBER
\STA\LINE\TO ONE
\JMP*\O$LE\RETURN
*
OUTL\DAC\**\OUTPUT LINE TO PRINTER
\STA\BUFR\A IS POINTER TO BUFFER
\LDA\=-10\INITIALIZE RETRY COUNTER
\STA\RCNT\FOR TEN ATTEMPTS
RTRY\LDA\BUFR\BUFFER ADDRESS
\STA\BPTR\INIT. POINTER
\LDA\HDSZ\CHARACTERS?
\TCA\\NEGATE
\STA\0
*
\SKS\LPNB\SKIP IF NOT BUSY
\JMP\*-1
\OCP\LPSS\START SCAN
OTL1\LDA*\BPTR\PICK UP CHARACTER PAIR
\OTA\LPCP\OUTPUT CHARACTER PAIR
\JMP\*-1\LOOP UNTIL READY
\IRS\BPTR\STEP POINTER
\IRS\0\STEP COUNTER
\JMP\OTL1\LOOP OVER CHARACTERS
*
\ARR\'20\SHORT TIMING DELAY?
\ARR\'03\11 MICROSECONDS (516)
*
\SKS\LPSR\SKIP IF READY
\SKP\\NOT YET - CHECK ALARM
\OCP\LPES\READY - STOP SCAN
OTL2\SKS\LPNA\SKIP IF NO ALARM
\JMP\ALRM\OOPS, SOMETHING WRONG
\SKS\LPNB\SKIP IF NOT BUSY
\JMP\OTL2\KEEP CHECKING
OUTX\LDX\SAVX\RESTORE X
\JMP*\OUTL\RETURN
*
ALRM\LDA\='52525\NOTICEABLE A-REG. PATTERN
\IRS\RCNT\STEP RETRY COUNTER
\JMP\RTRY\GIVE IT ANOTHER GO
\HLT\\OUT OF TRIES - HALT
\JMP\OUTX\RETURN IF RESTARTED
*
RCNT\DEC\-10\RETRY COUNTER
BUFR\BSS\1\BUFFER ADDRESS
BPTR\BSS\1\BUFFER WORD POINTER
SAVX\BSS\1\X SAVED HERE
*
*
HEAD\BCI\6,            \SPACES   1-12
    \BCI\6,            \SPACES  13-24
    \BCI\6,            \SPACES  25-36
    \BCI\6,            \SPACES  27-48
    \BCI\6,            \SPACES  49-60
    \BCI\6,            \SPACES  61-72
    \BCI\6,            \SPACES  73-84
    \BCI\6,            \SPACES  85-96
    \BCI\6, PAGE       \COLS    97-108
    \BCI\6,            \SPACES 109-120
HDRE\BCI\6,            \SPACES 121-132
* THESE EXTRA 6 WORDS (AT HDRE) WILL BE USED IF THE
* NUMBER OF WORDS IN THE HEADER (INDICATED BY O$LLHW)
* EXCEEDS 55 SINCE THEN 60 WORDS OF HEADER ARE COPIED,
* AND THE PAGE NUMBER IS NOT VISIBLE, BUT IT IS STILL
* COPIED INTO THE AREA AFTER THE 60 WORD HEADING, EACH
* TIME O$LE IS CALLED.
*
PAGN\BCI\3, PAGE \TO BE COPIED INTO BUFFER
\BCI\3,      \EXTRA SPACES - WHAT FOR?
*
*
* LITERALS
*
\FIN\\LITERALS
\END\\END OF O$LH 13.SEP.08 A.P.WISE
